@@ -1,7 +1,8 @@
 #!/usr/bin/env python
 
-## Copyright (C) 2008, 2009 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,12 +22,15 @@ from gettext import gettext as _
 import cupshelpers
 import urllib
 
+import ppdippstr
+
 class PhysicalDevice:
     def __init__(self, device):
         self.devices = None
         self._network_host = None
         self.add_device (device)
         self._user_data = {}
+        self._ppdippstr = ppdippstr.backends
 
     def _canonical_id (self, device):
         mfg = device.id_dict.get ('MFG', '')
@@ -41,11 +45,15 @@ class PhysicalDevice:
 
     def _get_host_from_uri (self, uri):
         (scheme, rest) = urllib.splittype (uri)
-        if scheme == 'hp':
+        if scheme == 'hp' or scheme == 'hpfax':
             if rest.startswith ("/net/"):
-                (hostport, rest) = urllib.splithost (rest[5:])
-                if hostport == None:
+                (rest, ipparam) = urllib.splitquery (rest[5:])
+                if ipparam != None and ipparam.startswith("ip="):
+                    hostport = ipparam[3:]
+                else:
                     return None
+            else:
+                return None
         else:
             (hostport, rest) = urllib.splithost (rest)
             if hostport == None:
@@ -56,9 +64,13 @@ class PhysicalDevice:
 
     def add_device (self, device):
         if self._network_host:
-            host = self._get_host_from_uri (device.uri)
+            if hasattr (device, 'address'):
+                host = device.address
+            else:
+                host = self._get_host_from_uri (device.uri)
+
             if host != self._network_host:
-                raise RuntimeError
+                raise ValueError
         else:
             (mfg, mdl) = self._canonical_id (device)
             if self.devices == None:
@@ -86,7 +98,7 @@ class PhysicalDevice:
 
                 sn = device.id_dict.get ('SN', '')
                 if sn != '' and self.sn != '' and sn != self.sn:
-                    raise RuntimeError
+                    raise ValueError
 
         for d in self.devices:
             if d.uri == device.uri:
@@ -107,7 +119,7 @@ class PhysicalDevice:
         # case of the hpfax backend), show the device-info field
         # instead.
         if self.mfg == '' or (self.mfg == "HP" and self.mdl == "Fax"):
-            return self.devices[0].info
+            return self._ppdippstr.get (self.devices[0].info)
 
         info = "%s %s" % (self.mfg, self.mdl)
         if len (self.sn) > 0:
@@ -169,7 +181,8 @@ class PhysicalDevice:
         mdlcmp = cmp (our_mdl.lower (), other_mdl.lower ())
         if mdlcmp != 0:
             return mdlcmp
-        if self.sn == '' or other.sn == '': return 0;
+        if self.sn == '' or other.sn == '':
+            return 0
         return cmp (self.sn, other.sn)
 
 if __name__ == '__main__':
