@@ -2,7 +2,7 @@
 
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009, 2010 Red Hat, Inc.
 ## Copyright (C) 2006 Florian Festi <ffesti@redhat.com>
 ## Copyright (C) 2006, 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
 
@@ -101,6 +101,11 @@ def ppdMakeModelSplit (ppd_make_and_model):
           l.startswith ("pagepro")):
         make = "KONICA MINOLTA"
         model = ppd_make_and_model
+    elif (l.startswith ("fs-") or
+          l.startswith ("km-") or
+          l.startswith ("taskalfa")):
+        make = "Kyocera Mita"
+        model = ppd_make_and_model
     elif l.startswith ("aficio"):
         make = "Ricoh"
         model = ppd_make_and_model
@@ -137,6 +142,15 @@ def ppdMakeModelSplit (ppd_make_and_model):
     elif l.startswith ("konica minolta "):
         make = "KONICA MINOLTA"
         model = ppd_make_and_model[15:]
+    elif l.startswith ("lexmark international "):
+        make = "Lexmark"
+        model = ppd_make_and_model[22:]
+    elif l.startswith ("kyocera mita "):
+        make = "Kyocera Mita"
+        model = ppd_make_and_model[13:]
+    elif l.startswith ("kyocera "):
+        make = "Kyocera Mita"
+        model = ppd_make_and_model[8:]
 
     # Finally, take the first word as the name of the manufacturer.
     else:
@@ -156,9 +170,6 @@ def ppdMakeModelSplit (ppd_make_and_model):
           makel.endswith ("packard")):
         make = "HP"
         makel = "hp"
-    elif makel == "lexmark international":
-        make = "Lexmark"
-        makel = "lexmark"
 
     # HP PPDs give NickNames like:
     # *NickName: "HP LaserJet 4 Plus v2013.111 Postscript (recommended)"
@@ -184,7 +195,12 @@ def ppdMakeModelSplit (ppd_make_and_model):
                    " ps3",
                    " pxl",
                    " series",
+                   " zjs",              # hpcups
+                   " zxs",              # hpcups
+                   " pcl3",             # hpcups
                    "_bt",
+                   " pcl",              # Canon CQue
+                   " ufr ii",           # Canon UFR II
                    ","]:
         s = modell.find (suffix)
         if s != -1:
@@ -216,13 +232,13 @@ DRIVER_TYPE_DOWNLOADED_NOW = 5
 DRIVER_TYPE_FOOMATIC_RECOMMENDED_NON_POSTSCRIPT = 8
 DRIVER_TYPE_VENDOR = 10
 DRIVER_TYPE_FOOMATIC_RECOMMENDED_POSTSCRIPT = 15
-DRIVER_TYPE_HPCUPS = 16
 DRIVER_TYPE_FOOMATIC_HPIJS_ON_HP = 17
 DRIVER_TYPE_GUTENPRINT_NATIVE_SIMPLIFIED = 20
 DRIVER_TYPE_GUTENPRINT_NATIVE = 25
 DRIVER_TYPE_SPLIX = 27
 DRIVER_TYPE_FOOMATIC_PS = 30
 DRIVER_TYPE_FOOMATIC_HPIJS = 40
+DRIVER_TYPE_HPCUPS = 45
 DRIVER_TYPE_FOOMATIC_GUTENPRINT_SIMPLIFIED = 50
 DRIVER_TYPE_FOOMATIC_GUTENPRINT = 60
 DRIVER_TYPE_FOOMATIC = 70
@@ -237,7 +253,12 @@ def _getDriverType (ppdname, ppds=None):
     if ppdname.find ("splix")!= -1:
         return DRIVER_TYPE_SPLIX
     if ppdname.find ("hpcups") != -1:
-        return DRIVER_TYPE_HPCUPS
+        info = ppds.getInfoFromPPDName (ppdname)
+        make_model = info.get ('ppd-make-and-model', '')
+        if make_model.find ("plugin") != -1:
+            return DRIVER_TYPE_3RD_PARTY_NONFREE
+        else:
+            return DRIVER_TYPE_HPCUPS
     if (ppdname.find (":") == -1 and
         ppdname.find ("/cups-included/") != -1):
         return DRIVER_TYPE_CUPS
@@ -245,14 +266,25 @@ def _getDriverType (ppdname, ppds=None):
         # Foomatic (generated) -- but which driver?
         if ppdname.find ("Generic")!= -1:
             return DRIVER_TYPE_FOOMATIC_GENERIC
-        if (ppds != None and
-            ppds.getInfoFromPPDName (ppdname).\
-            get ('ppd-make-and-model', '').find ("(recommended)") != -1):
-            if ppds.getInfoFromPPDName (ppdname).\
-               get ('ppd-make-and-model', '').find ("Postscript") != -1:
-                return DRIVER_TYPE_FOOMATIC_RECOMMENDED_POSTSCRIPT
-            else:
-                return DRIVER_TYPE_FOOMATIC_RECOMMENDED_NON_POSTSCRIPT
+        if ppds != None:
+            info = ppds.getInfoFromPPDName (ppdname)
+            device_id = info.get ('ppd-device-id', '')
+            id_dict = parseDeviceID (device_id)
+            drv = id_dict.get ('DRV', '')
+            drvfields = dict()
+            for field in drv.split (','):
+                if len (field) == 0:
+                    continue
+                key = field[0]
+                drvfields[key] = field[1:]
+
+            ppd_make_and_model = info.get ('ppd-make-and-model', '')
+            if (drvfields.get ('R', '0') == '1' or
+                ppd_make_and_model.find ("(recommended)") != -1):
+                if ppd_make_and_model.find ("Postscript") != -1:
+                    return DRIVER_TYPE_FOOMATIC_RECOMMENDED_POSTSCRIPT
+                else:
+                    return DRIVER_TYPE_FOOMATIC_RECOMMENDED_NON_POSTSCRIPT
         if ppdname.find ("-Postscript")!= -1:
             return DRIVER_TYPE_FOOMATIC_PS
         if ppdname.find ("-hpijs") != -1:
@@ -547,6 +579,10 @@ class PPDs:
             try:
                 ppdnamelist += self.ids["hp"][mdll]
                 status = self.STATUS_SUCCESS
+                print ("**** Incorrect IEEE 1284 Device ID: %s" %
+                       self.ids["hp"][mdll])
+                print "**** Actual ID is MFG:%s;MDL:%s;" % (mfg, mdl)
+                print "**** Please report a bug against the HPLIP component"
                 id_matched = True
             except KeyError:
                 pass
@@ -562,7 +598,8 @@ class PPDs:
             mdll = mdl.lower ()
 
         mfgrepl = {"hewlett-packard": "hp",
-                   "lexmark international": "lexmark"}
+                   "lexmark international": "lexmark",
+                   "kyocera": "kyocera mita"}
         if self.lmakes.has_key (mfgl):
             # Found manufacturer.
             make = self.lmakes[mfgl]
@@ -668,16 +705,18 @@ class PPDs:
         if not id_matched:
             sanitised_uri = re.sub (pattern="//[^@]*@/?", repl="//",
                                     string=str (uri))
-            print "No ID match for device %s:" % sanitised_uri
-            print "  <manufacturer>%s</manufacturer>" % mfg
-            print "  <model>%s</model>" % mdl
-            print "  <description>%s</description>" % description
             try:
                 cmd = reduce (lambda x, y: x + ","+ y, commandsets)
             except TypeError:
                 cmd = ""
+            id = "MFG:%s;MDL:%s;" % (mfg, mdl)
+            if cmd:
+                id += "CMD:%s;" % cmd
+            if description:
+                id += "DES:%s;" % description
 
-            print "  <commandset>%s</commandset>" % cmd
+            print "No ID match for device %s:" % sanitised_uri
+            print id
 
         print "Using %s (status: %d)" % (ppdnamelist[0], status)
         return (status, ppdnamelist[0])
@@ -825,15 +864,15 @@ class PPDs:
             return get ("PostScript Printer")
         elif (("pclxl" in cmdsets) or ("pcl-xl" in cmdsets) or
               ("pcl6" in cmdsets) or ("pcl 6 emulation" in cmdsets)):
-            return get ("PCL 6/PCL XL Printer")
+            return get ("PCL 6/PCL XL Printer", "PCL Laser Printer")
         elif "pcl5e" in cmdsets:
-            return get ("PCL 5e Printer")
+            return get ("PCL 5e Printer", "PCL Laser Printer")
         elif "pcl5c" in cmdsets:
-            return get ("PCL 5c Printer")
+            return get ("PCL 5c Printer", "PCL Laser Printer")
         elif ("pcl5" in cmdsets) or ("pcl 5 emulation" in cmdsets):
-            return get ("PCL 5 Printer")
+            return get ("PCL 5 Printer", "PCL Laser Printer")
         elif "pcl" in cmdsets:
-            return get ("PCL 3 Printer")
+            return get ("PCL 3 Printer", "PCL Laser Printer")
         elif (("escpl2" in cmdsets) or ("esc/p2" in cmdsets) or
               ("escp2e" in cmdsets)):
             return get ("ESC/P Dot Matrix Printer")
@@ -882,20 +921,10 @@ class PPDs:
             if not id:
                 continue
 
-            # Fix up broken Kyocera IDs
-            v = id.find (":Model")
-            if v != -1:
-                id = id[:v] + ';' + id[v + 1:]
-
             id_dict = parseDeviceID (id)
             lmfg = id_dict['MFG'].lower ()
             lmdl = id_dict['MDL'].lower ()
 
-            # Consider "HP" and "Hewlett-Packard" as equal, as the ID returned
-            # by the CUPS "usb" backend and HPLIP's "hp" backend are different
-            if lmfg == "hewlett-packard":
-                lmfg = "hp"
-
             bad = False
             if len (lmfg) == 0:
                 _debugprint ("Missing MFG field for %s" % ppdname)
@@ -1015,7 +1044,8 @@ def _self_test(argv):
         ("MFG:Canon;CMD:BJL,BJRaster3,BSCCe;SOJ:TXT01;MDL:iP3000;CLS:PRINTER;"
          "DES:Canon iP3000;VER:1.09;STA:10;FSI:03;", 1, "Canon PIXMA iP3000"),
         ("MFG:HP;MDL:Deskjet 5400 series;CMD:MLC,PCL,PML,DW-PCL,DESKJET,DYN;"
-         "1284.4DL:4d,4e,1;CLS:PRINTER;DES:5440;", 1, "HP DeskJet 5440"),
+         "1284.4DL:4d,4e,1;CLS:PRINTER;DES:5440;",
+         1, "HP DeskJet (5440|5550)"), # foomatic-db-hpijs used to say 5440
         ("MFG:Hewlett-Packard;MDL:HP LaserJet 3390;"
          "CMD:PJL,MLC,PCL,POSTSCRIPT,PCLXL;",
          0, "HP LaserJet 3390"),
@@ -1028,7 +1058,6 @@ def _self_test(argv):
         ("MFG:New;MDL:Unknown PCL5c Printer;CMD:PCL5c;", 2, "Generic PCL 5c"),
         ("MFG:New;MDL:Unknown PCL5 Printer;CMD:PCL5;", 2, "Generic PCL 5"),
         ("MFG:New;MDL:Unknown PCL3 Printer;CMD:PCL;", 2, "Generic PCL"),
-        ("MFG:New;MDL:Unknown ESC/P Printer;CMD:ESCP2E;", 2, "Generic ESC/P"),
         ("MFG:New;MDL:Unknown Printer;", 100, None),
         ]
 
