@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
-## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ def collect_printer_state_reasons (connection):
                 continue
             if not result.has_key (name):
                 result[name] = []
-            result[name].append (StateReason (name, reason))
+            result[name].append (StateReason (connection, name, reason))
     return result
 
 class Watcher:
@@ -208,7 +208,9 @@ class Monitor:
             self.connecting_timers[printer] = timer
             return False
 
-        del self.connecting_timers[printer]
+        if self.connecting_timers.has_key (printer):
+            del self.connecting_timers[printer]
+
         debugprint ("Still-connecting timer fired for `%s'" % printer)
         (printer_jobs, my_printers) = self.sort_jobs_by_printer ()
         self.update_connecting_devices (printer_jobs)
@@ -306,10 +308,10 @@ class Monitor:
                             break
 
                     if have_processing_job:
-                        t = gobject.timeout_add ((1 + CONNECTING_TIMEOUT)
-                                                 * 1000,
-                                                 self.check_still_connecting,
-                                                 printer)
+                        t = gobject.timeout_add_seconds (
+                            (1 + CONNECTING_TIMEOUT),
+                            self.check_still_connecting,
+                            printer)
                         self.connecting_timers[printer] = t
                         debugprint ("Start connecting timer for `%s'" %
                                     printer)
@@ -408,7 +410,7 @@ class Monitor:
                             break
                         if state_reason_is_harmless (reason):
                             continue
-                        reasons.append (StateReason (name, reason))
+                        reasons.append (StateReason (c, name, reason))
                     self.printer_state_reasons[name] = reasons
 
                     deferred_calls.append ((self.watcher.printer_event,
@@ -416,6 +418,12 @@ class Monitor:
                 continue
 
             # Job events
+            if not nse.startswith ("job-"):
+                # Some versions of CUPS give empty
+                # notify-subscribed-event attributes (STR #3608).
+                debugprint ("Unhandled nse %s" % repr (nse))
+                continue
+
             jobid = event['notify-job-id']
             if (nse == 'job-created' or
                 (nse == 'job-state-changed' and
@@ -481,9 +489,10 @@ class Monitor:
             if self.update_timer:
                 gobject.source_remove (self.update_timer)
 
-            interval = 1000 * notifications['notify-get-interval']
-            self.update_timer = gobject.timeout_add (interval,
-                                                     self.get_notifications)
+            interval = notifications['notify-get-interval']
+            t = gobject.timeout_add_seconds (interval,
+                                             self.get_notifications)
+            self.update_timer = t
 
         return False
 
@@ -527,7 +536,8 @@ class Monitor:
             events.extend (["job-created",
                             "job-completed",
                             "job-stopped",
-                            "job-state-changed"])
+                            "job-state-changed",
+                            "job-progress"])
 
         try:
             self.sub_id = c.createSubscription ("/", events=events)
@@ -536,8 +546,8 @@ class Monitor:
 
         cups.setUser (user)
 
-        self.update_timer = gobject.timeout_add (MIN_REFRESH_INTERVAL * 1000,
-                                                 self.get_notifications)
+        self.update_timer = gobject.timeout_add_seconds (MIN_REFRESH_INTERVAL,
+                                                         self.get_notifications)
         debugprint ("Created subscription %d" % self.sub_id)
 
         if self.monitor_jobs:
@@ -546,7 +556,8 @@ class Monitor:
                 # Filter out completed jobs.
                 filtered = {}
                 for jobid, job in jobs.iteritems ():
-                    if job['job-state'] < cups.IPP_JOB_CANCELED:
+                    if job.get ('job-state',
+                                cups.IPP_JOB_CANCELED) < cups.IPP_JOB_CANCELED:
                         filtered[jobid] = job
                 jobs = filtered
 
@@ -603,11 +614,26 @@ class Monitor:
             return False
 
         limit = 1
+        r = ["job-id",
+             "job-printer-uri",
+             "job-state",
+             "job-originating-user-name",
+             "job-k-octets",
+             "job-name",
+             "time-at-creation"]
         try:
-            fetched = c.getJobs (which_jobs=self.which_jobs,
-                                 my_jobs=self.my_jobs,
-                                 first_job_id=self.fetch_first_job_id,
-                                 limit=limit)
+            try:
+                fetched = c.getJobs (which_jobs=self.which_jobs,
+                                     my_jobs=self.my_jobs,
+                                     first_job_id=self.fetch_first_job_id,
+                                     limit=limit,
+                                     requested_attributes=r)
+            except TypeError:
+                # requested_attributes requires pycups 1.9.50
+                fetched = c.getJobs (which_jobs=self.which_jobs,
+                                     my_jobs=self.my_jobs,
+                                     first_job_id=self.fetch_first_job_id,
+                                     limit=limit)
         except cups.IPPError, (e, m):
             self.watcher.cups_ipp_error (self, e, m)
             self.fetch_jobs_timer = None
