@@ -2,8 +2,9 @@
 
 ## system-config-printer
 
-## Copyright (C) 2008, 2009 Red Hat, Inc.
-## Copyright (C) 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -24,7 +25,9 @@ from gettext import gettext as _
 printer_error_policy = dict()
 printer_op_policy = dict()
 job_sheets = dict()
+job_options = dict()
 ppd = dict()
+backends = dict()
 
 class TranslationDict:
     STR = {}
@@ -42,6 +45,7 @@ def init ():
     global printer_error_policy
     printer_error_policy = TranslationDict ({
             "abort-job": _("Abort job"),
+            "retry-current-job": _("Retry current job"),
             "retry-job": _("Retry job"),
             "stop-printer": _("Stop printer")
             })
@@ -65,6 +69,19 @@ def init ():
             "unclassified": _("Unclassified")
             })
 
+    # Names of job-hold-until values.
+    global job_options
+    job_options["job-hold-until"] = TranslationDict ({
+            "no-hold": _("No hold"),
+            "indefinite": _("Indefinite"),
+            "day-time": _("Daytime"),
+            "evening": _("Evening"),
+            "night": _("Night"),
+            "second-shift": _("Second shift"),
+            "third-shift": _("Third shift"),
+            "weekend": _("Weekend")
+            })
+
     ## Common PPD strings
 
     # Foomatic strings
@@ -143,3 +160,24 @@ def init ():
             "1200 dpi, Photo, Black + Color Cartr., Photo Paper":
                 _("1200 dpi, photo, black + color cartridge, photo paper"),
             })
+
+    ## Common backend descriptions
+    global backends
+    backends = TranslationDict ({
+            "Internet Printing Protocol (ipp)":
+                _("Internet Printing Protocol (ipp)"),
+            "Internet Printing Protocol (http)":
+                _("Internet Printing Protocol (http)"),
+            "Internet Printing Protocol (https)":
+                _("Internet Printing Protocol (https)"),
+            "LPD/LPR Host or Printer":
+                _("LPD/LPR Host or Printer"),
+            "AppSocket/HP JetDirect":
+                _("AppSocket/HP JetDirect"),
+            "Serial Port #1":
+                _("Serial Port #1"),
+            "LPT #1":
+                _("LPT #1"),
+            "Windows Printer via SAMBA":
+                _("Windows Printer via SAMBA"),
+            })
