@@ -1,7 +1,9 @@
 #!/usr/bin/env python
 
-## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
+##  Jiri Popelka <jpopelka@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,6 +19,9 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
+import cups
+import os
+
 _ = lambda x: x
 def set_gettext_function (fn):
     global _
@@ -33,11 +38,13 @@ class StateReason:
         ERROR: "dialog-error"
         }
 
-    def __init__(self, printer, reason):
+    def __init__(self, connection, printer, reason):
         self.printer = printer
         self.reason = reason
         self.level = None
         self.canonical_reason = None
+        self.connection = connection
+        self._ppd = None
 
     def get_printer (self):
         return self.printer
@@ -53,6 +60,7 @@ class StateReason:
             self.level = self.WARNING
         else:
             self.level = self.ERROR
+
         return self.level
 
     def get_reason (self):
@@ -69,6 +77,7 @@ class StateReason:
         return self.canonical_reason
 
     def __repr__ (self):
+        self.get_level()
         if self.level == self.REPORT:
             level = "REPORT"
         elif self.level == self.WARNING:
@@ -104,6 +113,10 @@ class StateReason:
                                      _("Printer '%s' may not be connected.")),
             'other': (_("Printer error"),
                       _("There is a problem on printer '%s'.")),
+
+            'cups-missing-filter': (_("Printer configuration error"),
+                                    _("There is a missing print filter for "
+                                      "printer '%s'.")),
             }
         try:
             (title, text) = messages[self.get_reason ()]
@@ -119,8 +132,31 @@ class StateReason:
                 title = _("Printer warning")
             elif self.get_level () == self.ERROR:
                 title = _("Printer error")
-            text = _("Printer '%s': '%s'.") % (self.get_printer (),
-                                               self.get_reason ())
+
+            if not self._ppd:
+                try:
+                    f = self.connection.getPPD(self.printer)
+                    self._ppd = cups.PPD (f)
+                    os.unlink (f)
+                except (cups.IPPError, RuntimeError, OSError):
+                    pass
+
+            reason = self.get_reason ()
+            if self._ppd:
+                try:
+                    schemes = ["text", "http", "help", "file"]
+                    localized_reason = ""
+                    for scheme in schemes:
+                        lreason = self._ppd.localizeIPPReason(self.reason,
+                                                              scheme)
+                        if lreason != None:
+                            localized_reason = localized_reason + lreason + ", "
+                    if localized_reason != "":
+                        reason = localized_reason[:-2]
+                except RuntimeError:
+                    pass
+
+            text = _("Printer '%s': '%s'.") % (self.get_printer (), reason)
         return (title, text)
 
     def get_tuple (self):
