@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "gnu" -*-
  * udev-configure-printer - a udev callout to configure print queues
- * Copyright (C) 2009 Red Hat, Inc.
+ * Copyright (C) 2009, 2010 Red Hat, Inc.
  * Author: Tim Waugh <twaugh@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 
 #include <cups/cups.h>
 #include <cups/http.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <libudev.h>
 #include <limits.h>
@@ -40,9 +41,11 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <syslog.h>
 #include <unistd.h>
 #include <usb.h>
+#include <glib.h>
 
 #define DISABLED_REASON "Unplugged or turned off"
 #define MATCH_ONLY_DISABLED 1
@@ -298,11 +301,18 @@ write_usb_uri_map (struct usb_uri_map *map)
   FILE *f;
 
   lseek (fd, SEEK_SET, 0);
-  ftruncate (fd, 0);
+  if (ftruncate (fd, 0) == -1)
+    {
+      syslog (LOG_ERR, "failed to ftruncate " USB_URI_MAP " (fd %d, errno %d)",
+	      fd, errno);
+      exit (1);
+    }
+
   f = fdopen (fd, "w");
   if (!f)
     {
-      syslog (LOG_ERR, "failed to fdopen " USB_URI_MAP " (fd %d)", fd);
+      syslog (LOG_ERR, "failed to fdopen " USB_URI_MAP " (fd %d, errno %d)",
+	      fd, errno);
       exit (1);
     }
 
@@ -311,11 +321,26 @@ write_usb_uri_map (struct usb_uri_map *map)
       size_t i;
       fprintf (f, "%s\t%s", entry->devpath, entry->uris.uri[0]);
       for (i = 1; i < entry->uris.n_uris; i++)
-	fprintf (f, "\t%s", entry->uris.uri[i]);
-      fwrite ("\n", 1, 1, f);
+	{
+	  if (fprintf (f, "\t%s", entry->uris.uri[i]) < 0)
+	    {
+	      syslog (LOG_ERR, "failed to fprintf " USB_URI_MAP " (errno %d)",
+		      errno);
+	      exit (1);
+	    }
+	}
+
+      if (fwrite ("\n", 1, 1, f) < 1)
+	{
+	  syslog (LOG_ERR, "failed to fwrite " USB_URI_MAP " (errno %d)",
+		  errno);
+	  exit (1);
+	}
     }
 
-  fclose (f);
+  if (fclose (f) == EOF)
+    syslog (LOG_ERR, "error closing " USB_URI_MAP " (errno %d)", errno);
+
   map->fd = -1;
 }
 
@@ -656,6 +681,36 @@ device_id_from_devpath (const char *devpath,
   return usb_device_devpath;
 }
 
+static void
+device_id_from_bluetooth (const char *bdaddr, struct device_id *id)
+{
+  gint exit_status;
+  char *device_id;
+  gchar *argv[4];
+
+  id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
+  argv[0] = g_strdup ("/usr/lib/cups/backend/bluetooth");
+  argv[1] = g_strdup ("--get-deviceid");
+  argv[2] = g_strdup (bdaddr);
+  argv[3] = NULL;
+  if (g_spawn_sync (NULL, argv, NULL,
+		    G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL,
+		    &device_id, NULL, &exit_status, NULL) == FALSE) {
+    g_free (argv[0]);
+    g_free (argv[1]);
+    g_free (argv[2]);
+    return;
+  }
+  g_free (argv[0]);
+  g_free (argv[1]);
+  g_free (argv[2]);
+
+  if (WEXITSTATUS(exit_status) == 0)
+    parse_device_id (device_id, id);
+
+  g_free (device_id);
+}
+
 static const char *
 no_password (const char *prompt)
 {
@@ -701,7 +756,6 @@ find_matching_device_uris (struct device_id *id,
   size_t i, n;
   const char *exclude_schemes[] = {
     "beh",
-    "bluetooth",
     "http",
     "https",
     "ipp",
@@ -1115,6 +1169,38 @@ enable_queue (const char *printer_uri, void *context)
   httpClose (cups);
 }
 
+static gboolean
+bluetooth_verify_address (const char *bdaddr)
+{
+  gboolean retval = TRUE;
+  char **elems;
+  guint i;
+
+  g_return_val_if_fail (bdaddr != NULL, FALSE);
+
+  if (strlen (bdaddr) != 17)
+    return FALSE;
+
+  elems = g_strsplit (bdaddr, ":", -1);
+  if (elems == NULL)
+    return FALSE;
+  if (g_strv_length (elems) != 6) {
+    g_strfreev (elems);
+    return FALSE;
+  }
+  for (i = 0; i < 6; i++) {
+    if (strlen (elems[i]) != 2 ||
+        g_ascii_isxdigit (elems[i][0]) == FALSE ||
+        g_ascii_isxdigit (elems[i][1]) == FALSE) {
+	  retval = FALSE;
+	  break;
+    }
+  }
+
+  g_strfreev (elems);
+  return retval;
+}
+
 static int
 do_add (const char *cmd, const char *devpath)
 {
@@ -1123,7 +1209,7 @@ do_add (const char *cmd, const char *devpath)
   struct device_id id;
   struct device_uris device_uris;
   struct usb_uri_map *map;
-  char *usb_device_devpath;
+  char *usb_device_devpath = NULL;
   char usbserial[256];
 
   if (getenv ("DEBUG") == NULL)
@@ -1151,8 +1237,13 @@ do_add (const char *cmd, const char *devpath)
   syslog (LOG_DEBUG, "add %s", devpath);
 
   map = read_usb_uri_map ();
-  usb_device_devpath = device_id_from_devpath (devpath, map, &id,
-					       usbserial, sizeof (usbserial));
+  if (bluetooth_verify_address (devpath)) {
+    usbserial[0] = '\0';
+    device_id_from_bluetooth (devpath, &id);
+  } else {
+    usb_device_devpath = device_id_from_devpath (devpath, map, &id,
+						 usbserial, sizeof (usbserial));
+  }
 
   if (!id.mfg || !id.mdl)
     {
@@ -1163,7 +1254,7 @@ do_add (const char *cmd, const char *devpath)
     }
 
   syslog (LOG_DEBUG, "MFG:%s MDL:%s SERN:%s serial:%s", id.mfg, id.mdl,
-	  id.sern ? id.sern : "-", usbserial);
+	  id.sern ? id.sern : "-", usbserial[0] ? usbserial : "-");
 
   find_matching_device_uris (&id, usbserial, &device_uris, usb_device_devpath,
 			     map);
@@ -1230,6 +1321,8 @@ disable_queue (const char *printer_uri, void *context)
 				     HTTP_ENCRYPT_IF_REQUESTED);
   ipp_t *request, *answer;
 
+  //FIXME remove instead of disable for Bluetooth
+
   if (cups == NULL)
     return;
 
